display_campaigns_url="/campaigns/api/display_list"

var campaigns_source =
{
    datatype: "json",
    datafields: [
        { name: 'id' },
        { name: 'title' },
        { name: 'start_date' },
        { name: 'end_date' },
        { name: 'status' }
    ],
    url: display_campaigns_url
};
var campaigns_dataAdapter = new $.jqx.dataAdapter(campaigns_source);

var selected_campaign;
var selected_row_index;

function campaigns_display_list() {
    $("#appWorkspace").append('<div id="campaigns_grid" class="campaigns_grid" bp="12"></div>');
    initialize_campaign_metrics_window();
    $("#campaigns_grid").jqxGrid({
            width: "95%",
            autoheight: true,
            sortable: true,
            pageable: true,
            selectionmode: 'singlerow',
            pagesizeoptions: ['5', '10', '15'],
            pagesize: 10,
            source: campaigns_dataAdapter,
            columns: [
                { text: 'Id', datafield: 'id', width: "5%" },
                { text: 'Title', datafield: 'title', width: "35%" },
                { text: 'Start Date', datafield: 'start_date', width: "20%" },
                { text: 'End Date', datafield: 'end_date', width: "20%" },
                { text: 'Status', datafield: 'status', width: "10%" },
                { text: 'Metrics', datafield: 'Edit', columntype: 'button', width: "10%", cellsrenderer: function () {
                     return "Metrics";
                  }, buttonclick: function (row) {
                    // get the clicked row's data
                    var dataRecord = $("#campaigns_grid").jqxGrid('getrowdata', row);
                    var campaign_id = dataRecord.id;

                    $.ajax({
                        url: '/campaigns/api/get_metrics/'+campaign_id,
                        success: function(data) {
                            display_campaign_metrics(dataRecord.title, data);
                        },
                        error: function(request,msg,error) {
                            display_ack_window("", "Error getting metrics for campaign "+campaign_id);
                        }
                    });

                }}
            ]
    });

    // adding the layout for all the editors...
    var editors_layout = [{
        type: 'layoutGroup',
        orientation: 'horizontal',
        width: '100%',
        items: [{
            type: 'documentGroup',
            width: '10%',
            items: [{
                type: 'documentPanel',
                title: 'Shifts',
                contentContainer: 'shiftsContainer'
            }]
        },{
            type: 'documentGroup',
            width: '30%',
            items: [{
                type: 'documentPanel',
                title: 'Targets',
                contentContainer: 'targetsContainer'
            }]
        },{
            type: 'documentGroup',
            width: '30%',
            items: [{
                type: 'documentPanel',
                title: 'Experiments',
                contentContainer: 'experimentsContainer'
            }]
        },{
            type: 'documentGroup',
            width: '30%',
            items: [{
                type: 'documentPanel',
                title: 'Templates Parameters',
                contentContainer: 'templatesContainer'
            }]
        }]
    }];

    $("#appWorkspace").append(`
        <div id="editorsLayout" bp=12 class="campaign_editor">
            <div data-container="shiftsContainer">
                <div id="shiftsEditor" class="config_editors"></div>
                <input type="button" id="saveShiftsConfig" class="save_button" />
            </div>
            <div data-container="targetsContainer">
                <div id="targetsEditor" class="config_editors"></div>
                <input type="button" id="saveTargetConfig" class="save_button" />
            </div>
            <div data-container="experimentsContainer">
                <div id="experimentsEditor" class="config_editors"></div>
                <input type="button" id="saveExperimentsConfig" class="save_button"/>
            </div>
            <div data-container="templatesContainer">
                <div id="templatesEditor" class="config_editors"></div>
                <input type="button" id="saveTemplatesConfig" class="save_button"/>
            </div>
        </div>
    `);

    $('#editorsLayout').jqxLayout({ layout: editors_layout });

    // 'text' or 'tree'
    var options = {
        mode: 'text'
    };
    var shifts_editor = new JSONEditor(document.getElementById("shiftsEditor"), options);
    var targets_editor = new JSONEditor(document.getElementById("targetsEditor"), options);
    var experiments_editor = new JSONEditor(document.getElementById("experimentsEditor"), options);
    var template_params_config_editor = new JSONEditor(document.getElementById("templatesEditor"), options);

    $('#saveShiftsConfig').jqxButton({ theme: 'darkblue', value: 'Save Configuration' });
    $('#saveTargetConfig').jqxButton({ theme: 'darkblue', value: 'Save Configuration' });
    $('#saveExperimentsConfig').jqxButton({ theme: 'darkblue', value: 'Save Configuration' });
    $('#saveTemplatesConfig').jqxButton({ theme: 'darkblue', value: 'Save Configuration' });

    document.getElementById('saveShiftsConfig').addEventListener('click', store_campaign_config("shifts", shifts_editor));
    document.getElementById('saveTargetConfig').addEventListener('click', store_campaign_config("resources", targets_editor));
    document.getElementById('saveExperimentsConfig').addEventListener('click', store_campaign_config("experiments", experiments_editor));
    document.getElementById('saveTemplatesConfig').addEventListener('click', store_campaign_config("template_params", template_params_config_editor));

    $('#campaigns_grid').on('rowselect', function (event) {
        var args = event.args;
        if (args) {
            var item = args.row;
            selected_campaign = item.id;
            selected_row_index = args.rowindex;
            $.ajax({
                url: '/campaigns/api/get_campaign_config/'+selected_campaign,
                success: function(result) {
                    shifts_editor.set(result['shifts'])
                    targets_editor.set(result['resources'])
                    experiments_editor.set(result['experiments'])
                    template_params_config_editor.set(result['template_params'])
                },
                error: function(request,msg,error) {
                    display_ack_window("", "Error getting configuration details for campaign: "+selected_campaign);
                }
            });
        } else {
            display_ack_window("", "Error selecting testing campaigns");
        };
    });
};

function refresh_campaign_config_params() {
    /* refreshes all editors */
    display_ack_window("Campaign configuration updated", "Configuration sections for campaign "+selected_campaign+" updated");
    $("#campaigns_grid").jqxGrid('selectrow', selected_row_index);
}

function store_campaign_config (field_name, editor) {
    return function() {
        var json = JSON.stringify(editor.get(), null, 2)

        p2store_update(
            table="test_campaigns",
            where={"id": selected_campaign},
            /* EcmaScript 2015 browsers should understand thay have to interpolate the
             * key...*/
            fields={ [field_name]: json},
            callback=refresh_campaign_config_params
        )
    }
}

function initialize_campaign_metrics_window() {
    var offset = $("#campaigns_grid").offset();
    $("#appWorkspace").append('<div id="campaign_metrics"><div id="metrics_window_title"></div><div id="metrics_window_content"></div></div>');
    $("#campaign_metrics").jqxWindow({ 
        position: { 
            x: parseInt(offset.left),
            y: parseInt(offset.top)
        },
        width: 820,
        height: 400,
        isModal: true, 
        autoOpen: false
    });

    $('#campaign_metrics').on('close', function (event) {
        $("#metrics_window_content").empty();
    });
}

function display_campaign_metrics(label, data) {
    $('#campaign_metrics').jqxWindow({title: `Campaign: ${label}`});
    add_total_tests(data);
    add_total_targets(data);

    $("#campaign_metrics").jqxWindow('open');
    add_bar_charts(data);
}

function add_total_tests(data) {
    $("#metrics_window_content").append(`
        <div bp="grid gap-none">
            <div class="metricsLabel" bp="3">Total number of tests: </div>
            <div bp="4" class="jqx-chart-title-text">${data.total_number_of_tests}</div>
        </div>
    `);
}

function add_total_targets(data) {
    $("#metrics_window_content").append(`
        <div bp="grid gap-none">
            <div class="metricsLabel" bp="3">Total number of targets: </div>
            <div bp="4" class="jqx-chart-title-text">${data.total_number_of_targets}</div>
        </div>
    `);
}

function add_bar_charts(data) {
    $('#metrics_window_content').append('<div id="metrics_bar_charts" bp="grid"></div>');

    add_target_coverage_barchart(data);
    add_execution_barchart(data);
    add_validation_barchart(data);
}

function add_target_coverage_barchart(data) {
    $('#metrics_bar_charts').append('<div bp="4" id="target_coverage_barchart"></div>');

    var settings = {
        title: "Target Coverage",
        description: "% of targets tested",
        showLegend: true,
        padding: { left: 5, top: 5, right: 5, bottom: 5 },
        titlePadding: { left: 5, top: 0, right: 0, bottom: 10 },
        source: [data['pp'][0]],
        xAxis: {
            dataField: 'Metric',
            showGridLines: true
        },
        seriesGroups: [
            {
                type: 'column',
                columnsGapPercent: 50,
                seriesGapPercent: 0,
                valueAxis: {
                    unitInterval: 10,
                    minValue: 0,
                    maxValue: 100,
                    displayValueAxis: true,
                    description: 'Percentage',
                    axisSize: 'auto',
                    tickMarksColor: '#888888'
                },
                series: [
                    { dataField: 'Coverage', displayText: 'Coverage'}
                ]
            }
        ]
    };
    $('#target_coverage_barchart').jqxChart(settings);
}

function add_execution_barchart(data) {
    $('#metrics_bar_charts').append('<div bp="3" id="execution_barchart"></div>');

    var settings = {
        title: "Tests status",
        description: "Distribution of tests per exit status",
        showLegend: true,
        enableAnimations: true,
        padding: { left: 5, top: 5, right: 5, bottom: 5 },
        titlePadding: { left: 5, top: 0, right: 0, bottom: 10 },
        source: [data['pp'][1]],
        xAxis: {
            dataField: 'Metric',
            showGridLines: true
        },
        valueAxis: {
            unitInterval: 10,
            minValue: 0,
            maxValue: 100,
            title: { text: 'Percentage' },
            labels: { horizontalAlignment: 'right' },
            tickMarks: { color: '#BCBCBC' }
        },
        colorScheme: 'scheme02',
        seriesGroups: [
            {
                type: 'stackedcolumn',
                columnsGapPercent: 50,
                seriesGapPercent: 0,
                series: [
                    { dataField: 'Aborted', displayText: 'Aborted', fillColor:'#D05011'},
                    { dataField: 'Completed', displayText: 'Completed', fillColor:'#90BF22'},
                    { dataField: 'Scheduled', displayText: 'Scheduled',fillColor:'#909090'},
                    { dataField: 'Deleted', displayText: 'Deleted', fillColor:'#DADADA'},
                ]
            }
        ]
    };
    $('#execution_barchart').jqxChart(settings);
}

function add_validation_barchart(data) {
    $('#metrics_bar_charts').append('<div bp="3" id="validation_barchart"></div>');

    var settings = {
        title: "Validation",
        description: "% of success/failure",
        showLegend: true,
        enableAnimations: true,
        padding: { left: 5, top: 5, right: 5, bottom: 5 },
        titlePadding: { left: 5, top: 0, right: 0, bottom: 10 },
        source: [data['pp'][2]],
        xAxis: {
            dataField: 'Metric',
            showGridLines: true
        },
        valueAxis: {
            unitInterval: 10,
            minValue: 0,
            maxValue: 100,
            title: { text: 'Percentage' },
            labels: { horizontalAlignment: 'right' },
            tickMarks: { color: '#BCBCBC' }
        },
        seriesGroups: [
            {
                type: 'stackedcolumn',
                columnsGapPercent: 50,
                seriesGapPercent: 0,
                series: [
                    { dataField: 'Passed', displayText: 'Passed', fillColor:'#90BF22'},
                    { dataField: 'Failed', displayText: 'Failed', fillColor:'#D05011'}
                ]
            }
        ]
    };
    $('#validation_barchart').jqxChart(settings);
}
